/* Doors for Linux.
   Copyright (C) 1998 Jason Lango

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2, or (at
   your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */

/*
**  test.h - Common test code.
**  Jason Lango <jal@rampant.org>
*/

#include <door.h>
#include <stdlib.h>
#include <string.h>
#include <signal.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdio.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <errno.h>
#include <sys/stat.h>
#include <stdarg.h>
#include <sys/mman.h>

#define VERIFY(c) \
	do { \
		if ((c) < 0) \
			test_error(#c ": %s\n", strerror(errno)); \
	} while(0)

#define DTS_MAX_CLIENTS		256
#define DTS_MAX_SERVERS		256

struct door_test_state {
	const char	*dts_testname;
	int		dts_nservers;
	int		dts_nservers_remain;
	int		dts_nclients;
	int		dts_nclients_remain;
	int		dts_client_pids[DTS_MAX_CLIENTS];
	int		dts_server_pids[DTS_MAX_SERVERS];
	int		dts_server_ready[DTS_MAX_SERVERS];
	int		dts_verbose;
};

/* Note that this structure is stored in memory shared by all of the
 * cooperating test processes started by the same process.
 */
extern struct door_test_state *dts;

#define TEST_COOKIE	((void*)0xfeedbeef)

typedef void (*test_server_proc_t)(void *cookie, char *argp, size_t arg_size,
	door_desc_t *dp, size_t n_desc);

/* These functions make it easier to write test code.
*/

int test_create_door(test_server_proc_t server_proc);
void test_destroy_door(void);
int test_open_door(void);
const char *test_door_filename(void);

void test_verify_info(struct door_info *info, test_server_proc_t server_proc);
void test_verify_cred(door_cred_t *cred);

void test_error(const char *fmt, ...);

int test_is_client_pid(int pid);
int test_is_server_pid(int pid);

void test_kill_servers(void);
void test_kill_clients(void);

/* These functions are provided by the test code.  Each client and server runs
 * in it's own process.
 */
const char *test_description(void);
int server_main(int num, int argc, char *argv[]);
int client_main(int num, int argc, char *argv[]);
